from django.contrib.sitemaps import GenericSitemap
from django.contrib.flatpages.sitemaps import FlatPageSitemap

from marcus import models


sitemaps = {
    'flatpages': FlatPageSitemap,
    'articles': GenericSitemap(
        {
            'queryset': models.Article.public.language(None).order_by('-published'),
            'date_field': 'published',
        },
        priority=0.9),
    'articles-en': GenericSitemap(
        {
            'queryset': models.Article.public.language('en').order_by('-published'),
            'date_field': 'published',
        },
        priority=0.9),
}



# coding: utf-8
from hashlib import md5

from django.core.urlresolvers import reverse
from django.template import loader, Template, Context, TemplateDoesNotExist
from django.views.decorators.http import condition
from django.contrib.syndication import views
from django.contrib.auth.models import User
from django.utils.feedgenerator import Atom1Feed
from django.utils import translation
from django.utils.translation import ugettext as _
from django.shortcuts import get_object_or_404
from django.conf import settings

from marcus import models, utils


class ContentAtom1Feed(Atom1Feed):
    def add_root_elements(self, handler):
        super(ContentAtom1Feed, self).add_root_elements(handler)
        hub_link = self.feed.get('hub_link')
        if hub_link is not None:
            handler.addQuickElement(u'link', '', {
                u'rel': u'hub',
                u'href': hub_link,
            })
        image_url = getattr(settings, 'MARCUS_FEED_IMAGE', '')
        handler.addQuickElement(u'logo', image_url, {})

    def add_item_elements(self, handler, item):
        super(ContentAtom1Feed, self).add_item_elements(handler, item)
        if item['content'] is not None:
            handler.addQuickElement(u'content', item['content'], {u'type': u'html', u'xml:base': item['link']})


def viewmethod(view):
    def method(self, request, *args, **kwargs):
        return view(request, self, *args, **kwargs)
    return method


class ContentFeed(views.Feed):
    feed_type = ContentAtom1Feed
    content_template = ''

    def hub_link(self, obj):
        pass

    def feed_extra_kwargs(self, obj):
        return {
            'hub_link': utils.absolute_url(self.hub_link(obj)),
        }

    def item_extra_kwargs(self, item):
        try:
            template = loader.get_template(self.content_template)
        except TemplateDoesNotExist:
            template = Template('{{ obj }}')
        return {
            'content': template.render(Context({'obj': item})),
        }

    def etag(self, request, *args, **kwargs):
        obj, language = self.get_object(request, *args, **kwargs)
        qs = self.get_queryset(obj, language)
        if not qs:
            return None
        return md5(str((self.updated(qs[0]), language))).hexdigest()

    @viewmethod
    @condition(lambda r, s, *args, **kwargs: s.etag(r, *args, **kwargs))
    def __call__(request, self, *args, **kwargs):
        return super(ContentFeed, self).__call__(request, *args, **kwargs)


class ArticleFeed(object):
    title_template = 'marcus/feeds/article_title.html'
    description_template = 'marcus/feeds/article_summary.html'
    content_template = 'marcus/feeds/article_content.html'
    author = User.objects.get(pk=settings.MARCUS_AUTHOR_ID)
    title = settings.MARCUS_TITLE
    subtitle = settings.MARCUS_SUBTITLE

    def author_name(self):
        return self.author.scipio_profile

    def items(self, (obj, language)):
        qs = self.get_queryset(obj, language)[:settings.MARCUS_ITEMS_IN_FEED]
        return [models.Translation(a, language) for a in qs]

    def item_pubdate(self, article):
        return article.published

    def updated(self, article):
        return article.published


class Article(ArticleFeed, ContentFeed):
    def get_object(self, request, language):
        translation.activate(language or 'ru')
        return None, language

    def link(self, (obj, language)):
        return utils.iurl(reverse('marcus-index'), language)

    def get_queryset(self, obj, language):
        return models.Article.public.language(language)


class Category(ArticleFeed, ContentFeed):
    def get_object(self, request, slug, language):
        translation.activate(language or 'ru')
        category = get_object_or_404(models.Category, slug=slug)
        return category, language

    def title(self, (category, language)):
        category = models.Translation(category, language)
        return u'%s » %s' % (settings.MARCUS_TITLE, category.title())

    def link(self, (category, language)):
        return utils.iurl(reverse('marcus-category', args=[category.slug]), language)

    def get_queryset(self, category, language):
        return models.Article.public.language(language).filter(categories=category)


class Tag(ArticleFeed, ContentFeed):
    def get_object(self, request, slug, language):
        translation.activate(language or 'ru')
        tag = get_object_or_404(models.Tag, slug=slug)
        return tag, language

    def title(self, (tag, language)):
        tag = models.Translation(tag, language)
        return u'%s » %s' % (settings.MARCUS_TITLE, tag.title())

    def link(self, (tag, language)):
        return utils.iurl(reverse('marcus-tag', args=[tag.slug]), language)

    def get_queryset(self, tag, language):
        return models.Article.public.language(language).filter(tags=tag)


class CommentFeed(object):
    title_template = 'marcus/feeds/comment_title.html'
    description_template = 'marcus/feeds/comment_summary.html'
    content_template = 'marcus/feeds/comment_content.html'
    subtitle = settings.MARCUS_SUBTITLE

    def title(self):
        return u'%s » %s' % (settings.MARCUS_TITLE, _(u'comments'))

    def items(self, (obj, language)):
        qs = self.get_queryset(obj, language).select_related(
            'author', 'author__scipio_profile', 'article', 'article__author'
        )[:settings.MARCUS_ITEMS_IN_FEED]
        return [models.Translation(c, language) for c in qs]

    def item_pubdate(self, comment):
        return comment.created

    def item_author_name(self, comment):
        return comment.author_str()

    def item_author_link(self, comment):
        if comment.type == 'comment':
            return comment.author_url()
        if comment.type == 'pingback':
            return comment.guest_url

    def updated(self, comment):
        return comment.approved


class Comment(CommentFeed, ContentFeed):
    def get_object(self, request, language):
        translation.activate(language or 'ru')
        return None, language

    def link(self, (obj, language)):
        return utils.iurl(reverse('marcus-index'), language)

    def get_queryset(self, obj, language):
        return models.Comment.public.language(language).order_by('-created')


class ArticleComment(CommentFeed, ContentFeed):
    def get_object(self, request, year, month, day, slug, language):
        translation.activate(language or 'ru')
        return get_object_or_404(
            models.Article,
            published__year=year,
            published__month=month,
            published__day=day,
            slug=slug,
        ), language

    def title(self, (article, language)):
        article = models.Translation(article, language)
        return u'%s » %s' % (article.title(), _(u'comments'))

    def link(self, (article, language)):
        article = models.Translation(article, language)
        return article.get_absolute_url()

    def get_queryset(self, article, language):
        return models.Comment.public.language(language).filter(article=article).order_by('-created')


class ArticleCommentShort(ArticleComment):
    def get_object(self, request, year, slug, language):
        translation.activate(language or 'ru')
        return get_object_or_404(
            models.Article,
            published__year=year,
            slug=slug,
        ), language



from django.db.utils import DatabaseError

from marcus import models


class ArticleTextSizeAdminMixin(object):
    max_text_sizes = models.Article.objects\
        .extra(select={'text_length': 'if(char_length(text_ru)>char_length(text_en),char_length(text_ru),char_length(text_en))'})\
        .order_by('-text_length')\
        .values('text_length')[0:5]

    def text_size(self, article):
        try:
            max_text_sizes = [i['text_length'] for i in self.max_text_sizes]
            max_text_size = int(sum(max_text_sizes) / len(max_text_sizes))
            percent = int((len(article.html()) * 100) / max_text_size)
        except (DatabaseError, ZeroDivisionError, ):
            # Because that in SQLite "if" there is no function.
            percent = 0
        return '<div style="width:100px;overflow:hidden"><div style="width:{percent}%;height:15px;background-color:#5B80B2">'\
               '</div></div>'.format(percent=int(percent) or 1)
    text_size.allow_tags = True



# coding: utf-8
import json
import scipio
import datetime
from scipio.forms import AuthForm

from django import http
from django.db.models import Q
from django.utils import timezone
from django.utils.safestring import mark_safe
from django.core.paginator import Paginator, InvalidPage
from django.core.urlresolvers import reverse
from django.views.decorators.http import require_POST
from django.shortcuts import render, redirect
from django.utils import translation
from django.contrib import auth
from django.conf import settings

from marcus import models, forms, antispam, utils
from marcus.utils import get_object_or_404


def object_list(request, queryset, template_name, context):
    paginator = Paginator(queryset, settings.MARCUS_PAGINATE_BY)
    try:
        page = paginator.page(int(request.GET.get('page', '1')))
    except (InvalidPage, ValueError):
        raise http.Http404()
    context.update({
        'object_list': page.object_list,
        'paginator': paginator,
        'page_obj': page,
    })
    return render(request, template_name, context)


def superuser_required(func):
    def wrapper(request, *args, **kwargs):
        if not request.user.is_authenticated() or not request.user.is_superuser:
            return http.HttpResponseForbidden('Superuser required', content_type='text/plain')
        return func(request, *args, **kwargs)
    return wrapper


def get_ip(request):
    ip = request.META.get('REMOTE_ADDR') or models.Comment._meta.get_field('ip').default
    if ip and ip.startswith('::ffff:'):
        ip = ip[7:]
    return ip


def index(request, language):
    translation.activate(language or 'ru')
    articles = models.Article.public.language(language)[:settings.MARCUS_ARTICLES_ON_INDEX]
    comments = list(models.Comment.public.language(language).order_by('-created')[:settings.MARCUS_COMMENTS_ON_INDEX])
    for comment in comments[:settings.MARCUS_COMMENT_EXCERPTS_ON_INDEX]:
        comment.show_excerpt = True

    return render(request, 'marcus/index.html', {
        'language': language,
        'essentials': models.Category.objects.filter(essential=True),
        'categories': models.Category.objects.popular(language),
        'tags': models.Tag.objects.popular(language),
        'articles': articles,
        'comments': comments,
        'meta_description': settings.MARCUS_DESCRIPTION,
        'meta_keywords': settings.MARCUS_KEYWORDS,
    })


def category_index(request, language):
    translation.activate(language or 'ru')
    return object_list(
        request,
        models.Category.objects.language(language),
        'marcus/category_list.html',
        {'language': language},
    )


def category(request, slug, language):
    translation.activate(language or 'ru')
    category = get_object_or_404(models.Category, slug=slug)
    return object_list(
        request,
        models.Article.public.language(language).filter(categories=category),
        'marcus/category.html',
        {
            'category': models.Translation(category, language),
            'language': language,
        },
    )


def tag_index(request, language):
    translation.activate(language or 'ru')
    return render(
        request,
        'marcus/tag_list.html',
        {'language': language, 'object_list': models.Tag.objects.language(language)},
    )


def tag(request, slug, language):
    translation.activate(language or 'ru')
    tag = get_object_or_404(models.Tag, slug=slug)
    return object_list(
        request,
        models.Article.public.language(language).filter(tags=tag),
        'marcus/tag.html',
        {
            'tag': models.Translation(tag, language),
            'language': language,
        },
    )


def archive_index(request, language):
    translation.activate(language or 'ru')
    settings.USE_TZ = False
    return render(request, 'marcus/archive-index.html', {
        'language': language,
        'months': sorted(models.Article.public.language(language).dates('published', 'month'), reverse=True),
    })


def archive(request, year, month, language):
    translation.activate(language or 'ru')
    queryset = models.Article.public.language(language)
    if month:
        try:
            first = datetime.datetime.strptime('%s-%s' % (year, month), '%Y-%m')
        except ValueError:
            raise http.Http404()
        queryset = queryset.filter(published__year=year, published__month=month)
    else:
        queryset = queryset.filter(published__year=year)
    return object_list(
        request,
        queryset,
        'marcus/archive.html',
        {
            'language': language,
            'year': year,
            'month': month and first,
        },
    )


def archive_short(request, year, language):
    args = [arg for arg in (year, language, ) if arg]
    url = reverse('marcus-archive', args=args)
    return redirect(url)


def find_article(request, slug):
    translation.activate('ru')
    objs = models.Article.public.filter(slug__startswith=slug)
    if not objs:
        raise http.Http404()
    if len(objs) == 1:
        return redirect(objs[0])
    else:
        return object_list(
            request,
            objs,
            'marcus/article_choice_list.html',
            {'slug': slug, 'language': None},
        )


def _process_new_comment(request, comment, language, check_login):
    spam_status = antispam.conveyor.validate(request, comment=comment)
    if spam_status == 'spam':
        comment.delete()
        return render(request, 'marcus/spam.html', {
            'text': comment.text,
            'admins': [e for n, e in settings.ADMINS],
        })
    if check_login and not request.user.is_authenticated():
        form = AuthForm(request.session, {'openid_identifier': request.POST['name']})
        if form.is_valid():
            comment = models.Translation(comment, language)
            url = form.auth_redirect(request, target=comment.get_absolute_url(), data={'acquire': str(comment.pk)})
            return redirect(url)
    comment.spam_status = spam_status
    if spam_status == 'clean':
        comment.approved = timezone.now()
    else:
        request.session['unapproved'] = comment.pk
    comment.save()
    return redirect(models.Translation(comment, language))


def article_short(request, year, slug, language):
    obj = get_object_or_404(models.Article, published__year=year, slug=slug)
    args = [i for i in year, obj.published.month, obj.published.day, slug, language if i]
    url = reverse('marcus-article', args=args)
    return redirect(url)


def article(request, year, month, day, slug, language):
    obj = get_object_or_404(
        models.Article, published__year=year, published__month=month, published__day=day, slug=slug
    )
    guest_name = request.session.get('guest_name', '')
    guest_email = request.session.get('guest_email', '')
    translation.activate(language or obj.only_language() or 'ru')
    if request.method == 'POST':
        form = forms.CommentForm(request.user, get_ip(request), obj, language, request.POST)
        if form.is_valid():
            request.session['guest_name'] = form.cleaned_data.get('name', '')
            request.session['guest_email'] = form.cleaned_data.get('xemail', '')
            form.cleaned_data['text'] = form.cleaned_data['text'].replace('script', u'sсript')
            comment = form.save()
            return _process_new_comment(request, comment, language, True)
    else:
        form = forms.CommentForm(article=obj, language=language)

    comments = models.Comment.common.language(language)\
        .select_related('author', 'author__scipio_profile', 'article')\
        .filter(article=obj, type="comment")\
        .filter(Q(guest_name=guest_name) | ~Q(approved=None))\
        .order_by('created', 'approved')

    unapproved = False
    try:
        unapproved_pk = request.session.get('unapproved')
        if unapproved_pk:
            unapproved = models.Comment.objects.get(pk=unapproved_pk, approved=None)
    except models.Comment.DoesNotExist:
        if 'unapproved' in request.session:
            del request.session['unapproved']

    retweet_url = u'http://twitter.com/home/?status={title}%20{url}%20{suffix}'.format(
        title=obj.title(),
        url=utils.absolute_url(utils.iurl(reverse('marcus-article-short', args=[obj.published.year, obj.slug, ]), language)),
        suffix=settings.MARCUS_RETWEET_SUFFIX.replace('@', '%40')
    )

    keywords = [tag.title(language) for tag in obj.tags.all()] +\
        [category.title(language) for category in obj.categories.all()]

    return render(request, 'marcus/article.html', {
        'article': models.Translation(obj, language),
        'comments': comments,
        # 'noteworthy_count': comments.filter(noteworthy=True).count(),
        'form': not obj.comments_hidden and form,
        'unapproved': unapproved,
        'language': language,
        'guest_name': guest_name,
        'guest_email': guest_email,
        'retweet_url': retweet_url,
        'meta_keywords': ", ".join(keywords),
        'meta_description': (obj.intro(language) or "").replace('"', "'"),
    })


@superuser_required
def draft(request, pk, language):
    translation.activate(language or 'ru')
    obj = get_object_or_404(models.Article, pk=pk)
    return render(request, 'marcus/article.html', {
        'article': models.Translation(obj, language),
        'categories': (models.Translation(c, language) for c in obj.categories.all()),
        'language': language,
    })


def acquire_comment(sender, user, acquire=None, language=None, **kwargs):
    if acquire is None:
        return
    try:
        auth.login(sender, user)
        comment = models.Comment.objects.get(pk=acquire)
        comment.author = user
        comment.save()
        _process_new_comment(sender, comment, language, False)
    except models.Comment.DoesNotExist:
        pass


@superuser_required
@require_POST
def approve_comment(request, id):
    comment = get_object_or_404(models.Comment, pk=id)
    antispam.conveyor.submit_ham(comment.spam_status, comment=comment)
    comment.approved = timezone.now()
    comment.save()
    if not comment.by_guest():
        scipio_profile = comment.author.scipio_profile
        if scipio_profile.spamer is None:
            scipio_profile.spamer = False
            scipio_profile.save()
    return redirect(spam_queue)


@superuser_required
@require_POST
def spam_comment(request, id):
    comment = get_object_or_404(models.Comment, pk=id)
    if not comment.by_guest():
        scipio_profile = comment.author.scipio_profile
        if scipio_profile.spamer is None:
            scipio_profile.spamer = True
            scipio_profile.save()
    antispam.conveyor.submit_spam(comment=comment)
    comment.delete()
    return redirect(comment.article)


@superuser_required
@require_POST
def delete_spam(request):
    models.Comment.suspected.all().delete()
    return redirect(spam_queue)


@superuser_required
def spam_queue(request):
    return object_list(
        request,
        models.Comment.suspected.select_related(),
        'marcus/spam_queue.html',
        {}
    )


@require_POST
def comment_preview(request):
    html = models.Comment(text=request.POST.get('text', '')).html()
    return http.HttpResponse(
        json.dumps({'status': 'valid', 'html': html}),
        content_type='application/json',
    )


def article_upload_image_preview(request, object_id):
    max_width = 300
    image_path = get_object_or_404(models.ArticleUpload, pk=object_id).upload.path

    import StringIO
    try:
        from PIL import Image
    except ImportError:
        import Image
    try:
        image = Image.open(image_path.encode('utf-8'))
    except Exception:
        return http.HttpResponse("Not a image", content_type="text/html")

    buffer = StringIO.StringIO()
    max_width = max_width if max_width < image.size[0] else image.size[0]
    height = int((float(image.size[1]) * float(max_width / float(image.size[0]))))
    image.resize((max_width, height), Image.ANTIALIAS).save(buffer, "PNG")
    return http.HttpResponse(buffer.getvalue(), content_type="image/png")


def search(request, language):
    SEARCH_LANGUAGES = (
        (None, translation.ugettext_lazy(u'All')),
        ('en', translation.ugettext_lazy(u'English')),
    )
    search_query = request.GET.get('s', '')
    search_language = request.GET.get('l', None)
    if search_language not in dict(SEARCH_LANGUAGES).keys():
        search_language = None

    language = search_language
    translation.activate(language or 'ru')

    return object_list(
        request,
        models.Article.public.language(language).search(search_query),
        'marcus/search.html',
        {
            'search_query': search_query,
            'search_language': search_language,
            'language': language,
            'search_languages': SEARCH_LANGUAGES,
        },
    )


def article_comments_unsubscribe(request, article_id, token, language):
    language = language or translation.get_language()
    translation.activate(language)

    article = get_object_or_404(models.Article, pk=article_id)
    comments = article.comments.filter(followup=True)
    for comment in comments:
        if comment.make_token() == token:
            found_comment = comment
            comments\
                .filter(guest_email=comment.guest_email)\
                .update(followup=False)
            break
    else:
        return http.HttpResponseForbidden(
            'Token is failed or expired!', content_type='text/plain')

    return render(request, 'marcus/unsubscribe.html', {
        'article_link': mark_safe(article.link(language)),
        'guest_name': found_comment.guest_name or found_comment.guest_email,
    })


scipio.signals.authenticated.connect(acquire_comment)



import re
import hashlib
import pytils
import markdown2
import itertools

from scipio.models import Profile

from django.db import models
from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.utils.safestring import mark_safe
from django.utils.text import Truncator
from django.utils.html import strip_tags
from django.utils.functional import curry
from django.utils.translation import ugettext as _
from django.db.models.signals import post_save

from marcus import utils
from marcus import managers


class Translation(object):
    def __init__(self, obj, language):
        super(Translation, self).__init__()
        self.obj = obj
        self.language = language

    def __getattr__(self, name):
        attr = getattr(self.obj, name)
        if getattr(attr, 'needs_language', None):
            attr = curry(attr, self.language)
        return attr

    def __dir__(self):
        return dir(self.obj)

    def __unicode__(self):
        """For drawing sequence

        Example:
        {% with article|translate:language as article %}
            Tags: {{ article.tags_links|safeseq|join:", " }}
        {% endwith %}
        """
        return unicode(self.obj)


class Category(models.Model):
    slug = models.SlugField(unique=True, blank=True)
    title_ru = models.CharField(max_length=255, blank=True)
    description_ru = models.TextField(blank=True)
    count_articles_ru = models.PositiveIntegerField(default=0)
    title_en = models.CharField(max_length=255, blank=True)
    description_en = models.TextField(blank=True)
    count_articles_en = models.PositiveIntegerField(default=0)
    parent = models.ForeignKey('self', null=True, blank=True)
    essential = models.BooleanField(default=False, db_index=True)

    objects = managers.CategoryManager()

    class Meta:
        verbose_name = 'category'
        verbose_name_plural = 'categories'

    def __unicode__(self):
        return self.title()

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = pytils.translit.slugify(self.title_en or self.title_ru)
        return self.save_base(*args, **kwargs)

    def title(self, language=None):
        if language:
            return self.title_en if language == 'en' else self.title_ru
        else:
            return self.title_ru or self.title_en
    title.needs_language = True

    def description(self, language=None):
        if language:
            return self.description_en if language == 'en' else self.description_ru
        else:
            return self.description_ru or self.description_en
    description.needs_language = True

    def get_absolute_url(self, language=None):
        return utils.iurl(reverse('marcus-category', args=[self.slug]), language)
    get_absolute_url.needs_language = True

    def get_feed_url(self, language=None):
        return utils.iurl('%sfeed/' % self.get_absolute_url(), language)
    get_feed_url.needs_language = True

    def article_count(self, language=None):
        return Article.public.language(language).filter(categories=self).count()
    article_count.needs_language = True

    def anchor(self, language=None):
        return u'<a href="{0}">{1}</a>'.format(self.get_absolute_url(language), self.title(language))
    anchor.needs_language = True


class Tag(models.Model):
    slug = models.SlugField(unique=True, blank=True)
    title_ru = models.CharField(max_length=255, blank=True)
    description_ru = models.TextField(blank=True)
    count_articles_ru = models.PositiveIntegerField(default=0)
    title_en = models.CharField(max_length=255, blank=True)
    description_en = models.TextField(blank=True)
    count_articles_en = models.PositiveIntegerField(default=0)

    objects = managers.TagManager()

    def __unicode__(self):
        return self.title()

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = pytils.translit.slugify(self.title_en or self.title_ru)

        if not self.title_en and utils.get_language_code_in_text(self.title_ru) == "en":
            self.title_en = self.title_ru

        if not self.title_ru:
            self.title_ru = self.title_en

        return self.save_base(*args, **kwargs)

    def get_absolute_url(self, language=None):
        return utils.iurl(reverse('marcus-tag', args=[self.slug]), language)
    get_absolute_url.needs_language = True

    def title(self, language=None):
        if language:
            return self.title_en if language == 'en' else self.title_ru
        else:
            return self.title_ru or self.title_en
    title.needs_language = True

    def article_count(self, language=None):
        return Article.public.language(language).filter(tags=self).count()
    article_count.needs_language = True

    def anchor(self, language=None):
        return u'<a href="{0}">{1}</a>'.format(self.get_absolute_url(language), self.title(language))
    anchor.needs_language = True

    def count(self, language=None):
        if language:
            count = self.count_articles_en if language == 'en' else self.count_articles_ru
        else:
            count = self.count_articles_ru or self.count_articles_en
        return count
    count.needs_language = True

    def size(self, language=None):
        return "{0}%".format(100 + (self.count(language) * 3))
    size.needs_language = True

    def css_class(self):
        return "tag_item"


class Article(models.Model):
    slug = models.SlugField(max_length=200, unique=True, blank=True)
    title_ru = models.CharField(max_length=255, blank=True)
    text_ru = models.TextField(blank=True)
    title_en = models.CharField(max_length=255, blank=True)
    text_en = models.TextField(blank=True)
    published = models.DateTimeField(blank=True, null=True, db_index=True)
    categories = models.ManyToManyField(Category)
    tags = models.ManyToManyField(Tag)
    updated = models.DateTimeField(auto_now=True)
    comments_hidden = models.BooleanField(default=False)

    objects = models.Manager()
    public = managers.PublicArticlesManager()

    def __unicode__(self):
        return self.slug

    def save(self, **kwargs):
        if not self.slug:
            self.slug = pytils.translit.slugify(self.title_ru or self.title_en)
        return super(Article, self).save(**kwargs)

    def only_language(self):
        return None if (self.text_en and self.text_ru) else 'en' if self.text_en else 'ru'

    def is_bilingual(self):
        return not self.only_language()

    def comment_language(self, display_language):
        if self.is_bilingual():
            return display_language or 'ru'
        return 'en' if self.text_en else 'ru'

    def get_absolute_url(self, language=None):
        if self.published:
            url = reverse('marcus-article', args=[
                '%04d' % self.published.year,
                '%02d' % self.published.month,
                '%02d' % self.published.day,
                self.slug
            ])
        else:
            url = reverse('marcus-draft', args=[self.pk])
        return utils.iurl(url, language)
    get_absolute_url.needs_language = True

    def get_translation_url(self, language=None):
        return self.get_absolute_url(utils.altlanguage(language))
    get_translation_url.needs_language = True

    def get_feed_url(self, language=None):
        return utils.iurl('%sfeed/' % self.get_absolute_url(), language)
    get_feed_url.needs_language = True

    def get_admin_url(self):
        viewname = "admin:{0}_{1}_change".format(
            self._meta.app_label,
            self._meta.model_name)
        return reverse(viewname, args=(self.pk, ))

    def title(self, language=None):
        if language:
            return self.title_en if language == 'en' else self.title_ru
        else:
            return self.title_ru or self.title_en
    title.needs_language = True

    def _language_text(self, language):
        if language:
            return self.text_en if language == 'en' else self.text_ru
        else:
            return self.text_ru or self.text_en

    def html(self, language=None):
        html = markdown2.markdown(self._language_text(language), extras=settings.MARCUS_MARKDOWN_EXTRAS)
        return mark_safe(html)
    html.needs_language = True

    def summary(self, language=None):
        return mark_safe(Truncator(strip_tags(self.html(language))).words(50))
    summary.needs_language = True

    def intro(self, language=None):
        result = markdown2.markdown(self._language_text(language), extras=settings.MARCUS_MARKDOWN_EXTRAS)
        pattern = re.compile(r'^(.*)<a name="more"></a>.*', re.S)
        match = re.match(pattern, result)
        return match and mark_safe(match.group(1))
    intro.needs_language = True

    def link(self, language=None):
        return u'<a href="{url}">{title}</a>'.format(
            url=self.get_absolute_url(language),
            title=self.title(language)
        )
    html.needs_language = True

    def full_link(self, language=None):
        current_site = Site.objects.get_current()
        url = "http://{domain}{url}".format(domain=current_site.domain, url=self.get_absolute_url(language))
        return u'<a href="{url}">{title}</a>'.format(url=url, title=self.title(language))
    html.needs_language = True

    def categories_links(self, language=None):
        return [category.anchor(language) for category in self.categories.all()]
    categories_links.needs_language = True

    def tags_links(self, language=None):
        return [tag.anchor(language) for tag in self.tags.all()]
    tags_links.needs_language = True


COMMENT_TYPES = (
    ('comment', _('Comment')),
)


LANGUAGES = (
    ('ru', _(u'Russian')),
    ('en', _(u'English')),
)


class ArticleUpload(models.Model):
    article = models.ForeignKey(Article, related_name="uploads")
    upload = models.FileField(upload_to="uploads")

    def __unicode__(self):
        return self.upload.name


class Comment(models.Model):
    article = models.ForeignKey(Article, related_name='comments')
    type = models.CharField(max_length=20, choices=COMMENT_TYPES)
    text = models.TextField(_(u'Text'))
    language = models.CharField(_(u'Language'), max_length=2, choices=LANGUAGES)
    author = models.ForeignKey(User)
    guest_name = models.CharField(max_length=255, blank=True)
    guest_email = models.CharField(max_length=200, blank=True, default='')
    guest_url = models.URLField(blank=True)
    ip = models.GenericIPAddressField(default='127.0.0.1')
    spam_status = models.CharField(max_length=20, blank=True, default='')
    created = models.DateTimeField(auto_now_add=True, db_index=True)
    approved = models.DateTimeField(null=True, blank=True, db_index=True)
    noteworthy = models.BooleanField(default=False)
    followup = models.BooleanField(
        help_text=_("Receive by email further comments in this conversation"),
        default=False, blank=True)

    objects = models.Manager()
    public = managers.PublicCommentsManager()
    suspected = managers.SuspectedCommentsManager()
    common = managers.CommentsManager()

    def make_token(self, salt="something"):
        secrets = map(unicode, [self.pk, self.guest_email, self.created, self.ip, salt])
        return hashlib.md5(u":".join(secrets)).hexdigest()

    def __unicode__(self):
        return u'%s, %s, %s' % (self.created.strftime('%Y-%m-%d'), self.article, self.author_str())

    def get_absolute_url(self, language=None):
        article = Translation(self.article, language)
        return '%s#comment-%s' % (article.get_absolute_url(), self.pk)
    get_absolute_url.needs_language = True

    def get_admin_url(self):
        viewname = "admin:{0}_{1}_change".format(
            self._meta.app_label,
            self._meta.model_name)
        return reverse(viewname, args=(self.pk, ))

    def html(self):
        html = markdown2.markdown(self.text, extras=settings.MARCUS_MARKDOWN_EXTRAS)
        return mark_safe(html)

    def summary(self):
        return mark_safe(Truncator(strip_tags(self.html())).words(50))

    def by_guest(self):
        return self.author.username == 'marcus_guest'

    def author_str(self):
        if self.by_guest():
            return self.guest_name
        try:
            return unicode(self.author.scipio_profile)
        except Profile.DoesNotExist:
            return unicode(self.author)

    def author_url(self):
        try:
            return self.author.scipio_profile.openid
        except (Profile.DoesNotExist, AttributeError):
            return None


def update_tag_and_category_counts(sender, instance, created, **kwargs):
    """Recalculation a counts for articles
    """
    objects = itertools.chain(instance.tags.all(), instance.categories.all())
    for o in objects:
        if instance.text_en:
            o.count_articles_en = o.article_count(language="en")
        if instance.text_ru:
            o.count_articles_ru = o.article_count(language="ru")
        o.save()


def notify_followers(sender, instance, created, **kwargs):
    if instance.approved:
        utils.notify_comment_followers(instance)
    if not instance.spam_status:
        utils.notify_comment_managers(instance)


post_save.connect(update_tag_and_category_counts, sender=Article)
post_save.connect(notify_followers, sender=Comment)



from django.db import models
from django.apps import apps
from django.conf import settings

MARCUS_SEARCH_FIELDS = ['title_ru', 'title_en', 'text_ru', 'text_en', ]
if hasattr(settings, 'MARCUS_SEARCH_FIELDS'):
    MARCUS_SEARCH_FIELDS = settings.MARCUS_SEARCH_FIELDS


class MarcusQuerySet(models.query.QuerySet):
    """Substitution the QuerySet
    """
    search_fields = MARCUS_SEARCH_FIELDS

    def search(self, search_query):
        if search_query:
            qq = models.Q()
            for field in self.search_fields:
                qq |= models.Q(**{"{0}__icontains".format(field): search_query})
            return self.filter(qq).distinct()
        return self.filter(pk__isnull=True)


class MarcusManager(models.Manager):
    def get_queryset(self):
        model = apps.get_model(self.model._meta.app_label, self.model._meta.object_name)
        return MarcusQuerySet(model)

    def __getattr__(self, attr, *args):
        try:
            return getattr(self.__class__, attr, *args)
        except AttributeError:
            return getattr(self.get_queryset(), attr, *args)



from django import forms
from django.utils.translation import ugettext as _
from django.contrib.auth.models import User

from marcus import models
from marcus.utils import get_language_code_in_text


def model_field(model, fieldname, **kwargs):
    return model._meta.get_field(fieldname).formfield(**kwargs)


class CommentForm(forms.Form):
    text = model_field(
        models.Comment, 'text', label=_(u'Text'), widget=forms.Textarea(attrs={'cols': '80', 'rows': '20'}))
    language = model_field(
        models.Comment, 'language', required=False)
    name = forms.CharField(
        label=_(u'Name or OpenID'), required=False)
    xemail = forms.EmailField(
        label=_(u'Email for notifications'), required=False)

    def __init__(self, user=None, ip=None, article=None, language=None, *args, **kwargs):
        super(CommentForm, self).__init__(*args, **kwargs)
        if user and not user.is_authenticated():
            user = User.objects.get(username='marcus_guest')
        self.user = user
        self.ip = ip
        self.article = article
        self.initial['language'] = article.comment_language(language)
        if not article.is_bilingual() or language:
            self.fields['language'].widget = forms.HiddenInput()

    def show_language(self):
        return not self.fields['language'].widget.is_hidden

    def clean_language(self):
        if not self.cleaned_data['language']:
            language = get_language_code_in_text(self.data.get('text', ''))
        else:
            raise forms.ValidationError(_(u'This field is required.'))
        return language

    def clean_name(self):
        if self.user.username != 'marcus_guest':
            return u''
        if not self.cleaned_data['name']:
            raise forms.ValidationError(_(u'This field is required.'))
        if User.objects.filter(username=self.cleaned_data['name']).exists():
            raise forms.ValidationError(_(u'This name is already taken.'))
        return self.cleaned_data['name']

    def save(self):
        guest_email = self.cleaned_data.get('xemail')
        return self.article.comments.create(
            type='comment',
            text=self.cleaned_data['text'],
            author=self.user,
            guest_name=self.cleaned_data.get('name'),
            guest_email=guest_email,
            ip=self.ip,
            language=self.cleaned_data['language'],
            followup=bool(guest_email),
        )



# coding: utf-8
import re
import os
import urllib2
import html2text
import urlparse

from django.core.urlresolvers import reverse, NoReverseMatch
from django.conf import settings

from marcus.wordpress_importer import BasePipeline


class EscapeTheUnderscore(BasePipeline):

    def replace_content(self, content):
        content = content.replace('_', '\_')
        items = []
        items.extend(re.findall('\<img[^\>]+\>', content, re.S | re.U | re.M))
        items.extend(re.findall('\<code[^\>]*\>.+?\<\/code\>', content, re.S | re.U | re.M))
        items.extend(re.findall('\<h\d{1}[^\>]*\>.+?\<\/h\d{1}\>', content, re.S | re.U | re.M))
        for item in items:
            new_item = item.replace('\_', '_')
            content = content.replace(item, new_item)
        return content


class BbCodeDetector(BasePipeline):

    def bb_caption_replace(self, content):
        """ Example: http://adw0rd.loc/2008/kupil-sebe-monitorchik/ru/
        [caption id="" align="alignnone" width="375" caption=" "].+[/caption]
        """
        content = re.sub('\[caption[^\]]+?\](.+?)\[\/caption\]', r'\1', content)
        return content

    def replace_content(self, content):
        if content.find('[caption') != -1:
            content = self.bb_caption_replace(content)
        return content


class CodecolorerToHighlightJsPipeline(BasePipeline):

    def replace_text(self, text):
        return self.replace_content(text)

    def replace_content(self, content):
        # From BB-code to html
        content = re.sub(r'\[cc([^\]]*)\]', r'<code\1>', content)
        content = content.replace('[/cc]', '</code>')

        # Convert </> to &lt;/&gt; into <code></code>
        blocks = []
        blocks.extend(re.findall(r'(\<code.*?\>)(.*?)(\<\/code\>)', content, re.M | re.U | re.S))
        # blocks.extend(re.findall(r'(\<blockquote.*?\>)(.*?)(\<\/blockquote\>)', content, re.M | re.U | re.S))
        for block in blocks:
            new_block = block[1].replace('<', '&lt;').replace('>', '&gt;')
            content = content.replace(block[1], new_block)

        # To highlight.js
        content = content.replace('<code lang=', '<code class=')
        content = content.replace('<code', '<pre><code')
        content = content.replace('</code>', '</code></pre>')

        return content


class WpContentUploadsToMediaPipeline(BasePipeline):

    ALLOW_DOMAINS = settings.MARCUS_WORDPRESS_IMPORTER.get('ALLOW_DOMAINS', tuple())

    def download_and_save(self, url):
        try:
            url_path = urlparse.urlparse(url).path
        except ValueError:
            return None

        file_path = url_path.replace('wp-content/uploads/', '')
        file_path = file_path.split('/')
        file_path, file_name = file_path[:-1], file_path[-1]
        media_root_path = os.path.join(settings.MEDIA_ROOT, *file_path)
        media_url_path = os.path.join(settings.MEDIA_URL, *file_path)

        try:
            os.makedirs(media_root_path)
        except OSError:
            pass

        try:
            # if settings.DEBUG is False:
            open(os.path.join(media_root_path, file_name), 'w').write(urllib2.urlopen(url).read())
        except (urllib2.HTTPError, urllib2.URLError), e:
            print e

        print "└─ Downloaded {url}".format(url=url)

        return os.path.join(media_url_path, file_name)

    def replace_content(self, content):
        domains = self.ALLOW_DOMAINS + ('', )  # Also available without the domain
        for domain in domains:
            scheme_and_host = 'http://{domain}'.format(domain=domain) if domain else ''
            url_paths = re.findall(
                'src\=\"{scheme_and_host}(\/wp-content\/uploads\/.+?)\"'.format(scheme_and_host=scheme_and_host),
                content) or []
            for url_path in url_paths:
                if not scheme_and_host:
                    scheme_and_host = 'http://{domain}'.format(domain=domains[0])
                url = scheme_and_host + url_path
                new_url = self.download_and_save(url)
                content = content.replace(url, new_url)
        return content


class ChangeUrlToArticleForImagePipeline(BasePipeline):
    """Change address link to article from /2012/slug-name/ to /2012/12/31/slug-name/
    """
    def replace_content(self, content):
        slug = self.data.get('slug')
        pub_date = self.data.get('pub_date')
        if slug and pub_date:
            # If you have something to change
            urls = re.findall('href\=\"(.+?\/{slug}\/)\"'.format(slug=slug), content) or []
            for url in urls:
                try:
                    new_url = reverse('marcus-article', args=[pub_date.year, pub_date.month, pub_date.day, slug])
                    content = content.replace(url, new_url)
                except NoReverseMatch:
                    pass
        return content


class RemoveImgClassPipeline(BasePipeline):
    """Remove .alignleft .wp-imaga and other classes into <img/>
    """

    def replace_content(self, content):
        for img in re.findall('\<img.+?(class\=\".+?\").+?\>', content, re.S | re.M) or []:
            content = content.replace(img, '')
        return content


class HtmlToMarkdownPipeline(BasePipeline):
    """Convert Html to Markdown. I recommended use it at the end pipelines
    """
    def replace_content(self, content):
        return html2text.html2text(content)


class ReplaceCommentedMoreToAnchored(BasePipeline):

    def replace_content(self, content):
        content = re.sub(r'<!--\s*more\s*-->', r'<a name="more"></a>', content)
        return content



from django.core.urlresolvers import reverse
from django.utils.encoding import smart_str

from scipio import antispam
from scipio.antispam import akismet
from scipio.models import Profile as ScipioProfile

from marcus import utils


class AkismetHandler(akismet.AkismetBaseHandler):
    def get_params(self, request, comment, **kwargs):
        try:
            openid = comment.author.scipio_profile.openid
        except ScipioProfile.DoesNotExist:
            openid = ''
        return {
            'blog': utils.absolute_url(reverse('marcus-index')),
            'user_ip': comment.ip,
            'permalink': utils.absolute_url(comment.get_absolute_url()),
            'comment_type': 'comment',
            'comment_author': smart_str(comment.by_guest() and comment.guest_name or comment.author),
            'comment_author_url': smart_str(openid),
            'comment_content': smart_str(comment.text),
        }


class Paranoia(object):
    def validate(self, request, **kwargs):
        return 'paranoia'

conveyor = antispam.Conveyor([
    antispam.WhitelistHandler(),
    antispam.HoneyPotHandler(),
    AkismetHandler(),
    Paranoia(),
])



from django.core.urlresolvers import reverse
from django.contrib.admin.widgets import AdminFileWidget
from django.utils.safestring import mark_safe


class AdminImageWidget(AdminFileWidget):
    """
    A FileField Widget that displays an image instead of a file path
    if the current file is an image.
    """
    def render(self, name, value, attrs=None):
        output = []
        if value:
            url = value.url if hasattr(value, 'url') else value.name
            thumbnail_url = reverse('article-upload-image-preview', args=[value.instance.pk])
            output.append(
                '<a href="{url}"><img src="{thumbnail_url}" /></a><br />'
                '<tt>{url}</tt><br />'.format(url=url, thumbnail_url=thumbnail_url))
        output.append(super(AdminFileWidget, self).render(name, value, attrs))
        return mark_safe("".join(output))



from django.db import models
from django.conf import settings

from marcus.queryset import MarcusManager


class CommonLanguageManager(MarcusManager):

    def language(self, code):
        queryset = self.get_queryset()
        if code == 'en':
            queryset = queryset.exclude(title_en='')
        elif code == 'ru':
            queryset = queryset.exclude(title_ru='')
        return queryset


class PublicArticlesManager(CommonLanguageManager):
    def get_queryset(self):
        return super(PublicArticlesManager, self).get_queryset()\
            .exclude(published=None).order_by('-published')


class CategoryManager(CommonLanguageManager):
    def language(self, code):
        queryset = super(CategoryManager, self).language(code)
        queryset = queryset.order_by('title_en' if code == 'en' else 'title_ru')
        return queryset

    def popular(self, code):
        order_by = ['-essential']
        queryset = self.get_queryset()
        if code == "en":
            order_by.append('-count_articles_en')
            queryset = queryset.filter(count_articles_en__gt=0)
        else:
            order_by.extend(['-count_articles_ru', '-count_articles_en', ])
            queryset = queryset.filter(models.Q(count_articles_ru__gt=0) | models.Q(count_articles_en__gt=0))
        return queryset.order_by(*order_by)


class TagManager(CategoryManager):
    def popular(self, code):
        order_by = []
        queryset = self.get_queryset()
        if code == "en":
            order_by.append('-count_articles_en')
            queryset = queryset.filter(count_articles_en__gt=0)
        else:
            order_by.extend(['-count_articles_ru', '-count_articles_en', ])
            queryset = queryset.filter(
                models.Q(count_articles_ru__gt=settings.MARCUS_TAG_MINIMUM_ARTICLES) |
                models.Q(count_articles_en__gt=settings.MARCUS_TAG_MINIMUM_ARTICLES))
        return queryset.order_by(*order_by)


class CommentsManager(models.Manager):
    def language(self, code):
        queryset = self.get_queryset()
        if code:
            queryset = queryset.filter(language=code)
        return queryset


class PublicCommentsManager(CommentsManager):
    def get_queryset(self):
        return super(PublicCommentsManager, self).get_queryset()\
            .exclude(approved=None).order_by('created')


class SuspectedCommentsManager(models.Manager):
    def get_queryset(self):
        return super(SuspectedCommentsManager, self).get_queryset()\
            .filter(approved=None).exclude(spam_status='clean')\
            .order_by('-created')



# coding: utf-8
import re
import os
import sys
from urllib2 import urlopen
from datetime import datetime
from optparse import make_option
from xml.etree import ElementTree as ET

from django.conf import settings
from django.utils import timezone
from django.core.files import File
from django.utils.text import Truncator
from django.utils.html import strip_tags
from django.db.utils import IntegrityError
from django.utils.encoding import smart_str
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.management.base import CommandError
from django.core.management.base import LabelCommand
from django.core.files.temp import NamedTemporaryFile
from django.utils.importlib import import_module

from marcus.models import Article, Tag, Category, Comment
# from marcus.utils import get_language_code_in_text

MARCUS_GUEST = User.objects.get(username="marcus_guest")
PINGBACK = 'pingback'
TRACKBACK = 'trackback'

if settings.DEBUG is True:
    # Cleanup models
    Article.objects.all().delete()
    Category.objects.all().delete()
    Tag.objects.all().delete()
    Comment.objects.all().delete()

WP_NS = 'http://wordpress.org/export/%s/'
ARTICLE_PIPELINES = settings.MARCUS_WORDPRESS_IMPORTER.get('ARTICLE_PIPELINES', tuple())
CATEGORY_PIPELINES = settings.MARCUS_WORDPRESS_IMPORTER.get('CATEGORY_PIPELINES', tuple())
TAG_PIPELINES = settings.MARCUS_WORDPRESS_IMPORTER.get('TAG_PIPELINES', tuple())
COMMENT_PIPELINES = settings.MARCUS_WORDPRESS_IMPORTER.get('COMMENT_PIPELINES', tuple())


class Command(LabelCommand):
    """Command for importing a WordPress blog into Marcus via a WordPress Export file."""
    help = 'Import a Wordpress blog into Marcus.'
    label = 'WXR file'
    args = 'wordpress.xml'

    option_list = LabelCommand.option_list + (
        make_option(
            '--noautoexcerpt',
            action='store_false',
            dest='auto_excerpt',
            default=True,
            help='Do NOT generate an excerpt if not present.'),
        make_option(
            '--author',
            dest='author',
            default='',
            help='All imported articles belong to specified author'))

    SITE = Site.objects.get_current()

    def __init__(self):
        """Init the Command and add custom styles"""
        super(Command, self).__init__()
        self.style.TITLE = self.style.SQL_FIELD
        self.style.STEP = self.style.SQL_COLTYPE
        self.style.ITEM = self.style.HTTP_INFO

    def write_out(self, message, verbosity_level=1):
        """Convenient method for outputing"""
        if self.verbosity and self.verbosity >= verbosity_level:
            sys.stdout.write(smart_str(message))
            sys.stdout.flush()

    def handle_label(self, wxr_file, **options):
        global WP_NS
        self.verbosity = int(options.get('verbosity', 1))
        self.auto_excerpt = options.get('auto_excerpt', True)
        self.default_author = options.get('author')
        if self.default_author:
            try:
                self.default_author = User.objects.get(username=self.default_author)
            except User.DoesNotExist:
                raise CommandError('Invalid username for default author')

        self.write_out(self.style.TITLE('Starting migration from Wordpress to Marcus!\n'))

        tree = ET.parse(wxr_file)
        WP_NS = WP_NS % self.guess_wxr_version(tree)

        self.authors = self.import_authors(tree)
        self.categories = self.import_categories(tree.findall('channel/{%s}category' % WP_NS))
        self.tags = self.import_tags(tree.findall('channel/{%s}tag' % WP_NS))
        self.articles = self.import_articles(tree.findall('channel/item'))

    def guess_wxr_version(self, tree):
        """We will try to guess the wxr version used
        to complete the wordpress xml namespace name"""
        for v in ('1.2', '1.1', '1.0'):
            try:
                tree.find('channel/{%s}wxr_version' % (WP_NS % v)).text
                return v
            except AttributeError:
                pass
        raise CommandError('Cannot resolve the wordpress namespace')

    def import_authors(self, tree):
        """Retrieve all the authors used in posts
        and convert it to new or existing user, and
        return the convertion"""
        self.write_out(self.style.STEP('- Importing authors\n'))
        post_authors = set()
        for item in tree.findall('channel/item'):
            post_type = item.find('{%s}post_type' % WP_NS).text
            if post_type == 'post':
                post_authors.add(item.find('{http://purl.org/dc/elements/1.1/}creator').text)

        self.write_out('> %i authors found.\n' % len(post_authors))

        authors = {}
        for post_author in post_authors:
            if self.default_author:
                authors[post_author] = self.default_author
            else:
                authors[post_author] = self.migrate_author(post_author)
        return authors

    def migrate_author(self, author_name):
        """Handle actions for migrating the users"""
        action_text = "The author '%s' needs to be migrated to an User:\n"\
                      "1. Use an existing user ?\n"\
                      "2. Create a new user ?\n"\
                      "Please select a choice: " % self.style.ITEM(author_name)
        while 42:
            selection = raw_input(smart_str(action_text))
            if selection and selection in '12':
                break
        if selection == '1':
            users = User.objects.all()
            if users.count() == 1:
                preselected_user = None
                usernames = ['[%s]' % users[0].username]
            else:
                usernames = []
                preselected_user = None
                for user in users:
                    if user.username == author_name:
                        usernames.append('[%s]' % user.username)
                        preselected_user = author_name
                    else:
                        usernames.append(user.username)
            while 42:
                user_text = "1. Select your user, by typing " \
                            "one of theses usernames:\n"\
                            "%s or 'back'\n"\
                            "Please select a choice: " % ', '.join(usernames)
                user_selected = raw_input(user_text)
                if user_selected in usernames:
                    break
                if user_selected == '' and preselected_user:
                    user_selected = preselected_user
                    break
                if user_selected.strip() == 'back':
                    return self.migrate_author(author_name)
            return users.get(username=user_selected)
        else:
            create_text = "2. Please type the email of " \
                          "the '%s' user or 'back': " % author_name
            author_mail = raw_input(create_text)
            if author_mail.strip() == 'back':
                return self.migrate_author(author_name)
            try:
                return User.objects.create_user(author_name, author_mail)
            except IntegrityError:
                return User.objects.get(username=author_name)

    def import_categories(self, category_nodes):
        """Import all the categories from 'wp:category' nodes,
        because categories in 'item' nodes are not necessarily
        all the categories and returning it in a dict for
        database optimizations."""
        self.write_out(self.style.STEP('- Importing categories\n'))

        categories = {}
        for category_node in category_nodes:
            category_data = {}
            title = category_node.find('{%s}cat_name' % WP_NS).text[:255]
            slug = category_node.find('{%s}category_nicename' % WP_NS).text[:255]
            try:
                parent_title = category_node.find('{%s}category_parent' % WP_NS).text[:255]
            except TypeError:
                parent_title = None

            category_data = {'title': title, 'slug': slug, 'parent_title': parent_title}
            self.write_out(u'> {title}... '.format(title=title))

            for pipeline_path in CATEGORY_PIPELINES:
                category_data = self._get_pipeline(pipeline_path)(data=category_data).output()

            category, created = Category.objects.get_or_create(
                title_ru=category_data.get('title'),
                slug=category_data.get('slug'),
                parent=categories.get(parent_title))

            categories[title] = category

            self.write_out(self.style.ITEM('OK\n'))

        return categories

    def import_tags(self, tag_nodes):
        """Import all the tags form 'wp:tag' nodes,
        because tags in 'item' nodes are not necessarily
        all the tags, then use only the nicename, because it's like
        a slug and the true tag name may be not valid for url usage."""

        self.write_out(self.style.STEP('- Importing tags\n'))

        tags = {}
        for tag_node in tag_nodes:
            name = tag_node.find('{%s}tag_name' % WP_NS).text[:50]
            slug = tag_node.find('{%s}tag_slug' % WP_NS).text[:50]

            tag_data = {'name': name, 'slug': slug}
            self.write_out(u'> {name}... '.format(name=name))

            for pipeline_path in TAG_PIPELINES:
                tag_data = self._get_pipeline(pipeline_path)(data=tag_data).output()

            tag_name = tag_data.get('name')
            is_english_tag = bool(re.search(u'[^а-яА-Я]+', tag_name))

            tag_ = dict(title_ru=tag_data.get('name'), slug=tag_data.get('slug'))
            if is_english_tag:
                tag_.update(dict(title_en=tag_name))

            tag, created = Tag.objects.get_or_create(**tag_)

            tags[tag_name] = tag
            self.write_out(self.style.ITEM('OK\n'))

        return tags

    def get_article_tags(self, category_nodes):
        """Return a list of article's tags,
        by using the nicename for url compatibility"""
        tags = []
        for category_node in category_nodes:
            domain = category_node.attrib.get('domain')
            if domain == 'post_tag':
                tags.append(category_node.text)
        return tags

    def get_article_categories(self, category_nodes):
        """Return a list of article's categories
        based of imported categories"""
        categories = []
        for category_node in category_nodes:
            domain = category_node.attrib.get('domain')
            if domain == 'category':
                categories.append(category_node.text)
        return categories

    def import_article(self, title, content, item_node):
        """Importing an article but some data are missing like
        related articles, start_publication and end_publication.
        start_publication and creation_date will use the same value,
        wich is always in Wordpress $post->post_date"""
        creation_date = datetime.strptime(item_node.find('{%s}post_date' % WP_NS).text, '%Y-%m-%d %H:%M:%S')
        pub_date = datetime.strptime(item_node.find('pubDate').text, '%a, %d %b %Y %H:%M:%S +0000')
        if settings.USE_TZ:
            creation_date = timezone.make_aware(creation_date, timezone.utc)
            pub_date = timezone.make_aware(pub_date, timezone.utc)

        excerpt = item_node.find('{%sexcerpt/}encoded' % WP_NS).text
        if not excerpt:
            if self.auto_excerpt:
                excerpt = Truncator('...').words(50, strip_tags(content))
            else:
                excerpt = ''

        article_data = {
            'title': title,
            'content': content,
            'excerpt': excerpt,
            # Prefer use this function than
            # item_node.find('{%s}post_name' % WP_NS).text
            # Because slug can be not well formated
            # 'slug': slugify(title)[:255] or 'post-%s' % item_node.find('{%s}post_id' % WP_NS).text,
            'slug': item_node.find('{%s}post_name' % WP_NS).text,
            'categories': self.get_article_categories(item_node.findall('category')),
            'tags': self.get_article_tags(item_node.findall('category')),
            'status': item_node.find('{%s}status' % WP_NS).text,
            'comment_enabled': item_node.find('{%s}comment_status' % WP_NS).text == 'open',
            'pingback_enabled': item_node.find('{%s}ping_status' % WP_NS).text == 'open',
            'creation_date': creation_date,
            'pub_date': pub_date,
        }

        return article_data

    def import_articles(self, items):
        """Loops over items and find article to import,
        an article need to have 'post_type' set to 'post' and
        have content."""
        self.write_out(self.style.STEP('- Importing articles\n'))

        for item_node in items:
            title = (item_node.find('title').text or '')[:255]
            post_type = item_node.find('{%s}post_type' % WP_NS).text
            content = item_node.find('{http://purl.org/rss/1.0/modules/content/}encoded').text

            if post_type == 'post' and title:
                article_data = self.import_article(title, content, item_node)

                for pipeline_path in ARTICLE_PIPELINES:
                    article_data = self._get_pipeline(pipeline_path)(data=article_data).output()

                status = article_data.get('status')
                self.write_out(u'> {title}, {status}... '.format(title=title, status=status))

                article, created = Article.objects.get_or_create(
                    slug=article_data.get('slug'),
                    title_ru=article_data.get('title'),
                    text_ru=article_data.get('content'),
                    published=article_data.get('pub_date') if status == "publish" else None,
                    updated=article_data.get('creation_date'),
                    comments_hidden=not article_data.get('comment_enabled'))

                for category_name in article_data.get('categories'):
                    category = self.categories.get(category_name)
                    if category:
                        article.categories.add(category)

                for tag_name in article_data.get('tags'):
                    tag = self.tags.get(tag_name)
                    if tag:
                        article.tags.add(tag)

                # article.authors.add(self.authors[item_node.find('{http://purl.org/dc/elements/1.1/}creator').text])
                # article.sites.add(self.SITE)

                self.write_out(self.style.ITEM('OK\n'))

                image_id = self.find_image_id(item_node.findall('{%s}postmeta' % WP_NS))
                if image_id:
                    self.import_image(article, items, image_id)

                self.import_comments(article, item_node.findall('{%s}comment' % WP_NS))
            else:
                self.write_out(u'> {title}... '.format(title=title), 2)
                self.write_out(self.style.NOTICE('SKIPPED (not a post)\n'), 2)

    def find_image_id(self, metadatas):
        for meta in metadatas:
            if meta.find('{%s}meta_key' % WP_NS).text == '_thumbnail_id':
                return meta.find('{%s}meta_value/' % WP_NS).text

    def import_image(self, article, items, image_id):
        for item in items:
            post_type = item.find('{%s}post_type' % WP_NS).text
            if post_type == 'attachment' and item.find('{%s}post_id' % WP_NS).text == image_id:

                title = u'Attachment {title}'.format(title=item.find('title').text)
                self.write_out(u' > {title}... '.format(title=title))

                image_url = item.find('{%s}attachment_url' % WP_NS).text
                img_tmp = NamedTemporaryFile(delete=True)
                img_tmp.write(urlopen(image_url).read())
                img_tmp.flush()
                article.image.save(os.path.basename(image_url), File(img_tmp))

                self.write_out(self.style.ITEM('OK\n'))

    def import_comments(self, article, comment_nodes):
        """Loops over comments nodes and import then
        in django.contrib.comments"""
        for comment_node in comment_nodes:
            is_pingback = comment_node.find('{%s}comment_type' % WP_NS).text == PINGBACK
            # is_trackback = comment_node.find('{%s}comment_type' % WP_NS).text == TRACKBACK

            comment_data = {}
            comment_data['article'] = article
            comment_data['id'] = comment_node.find('{%s}comment_id' % WP_NS).text
            comment_data['type'] = "pingback" if is_pingback else "comment"
            comment_data['text'] = comment_node.find('{%s}comment_content' % WP_NS).text
            comment_data['language'] = 'ru'  # get_language_code_in_text(comment_data['text'] or '')

            comment_data['guest_name'] = comment_node.find('{%s}comment_author' % WP_NS).text or ''
            comment_data['guest_email'] = comment_node.find('{%s}comment_author_email' % WP_NS).text or ''
            comment_data['guest_url'] = comment_node.find('{%s}comment_author_url' % WP_NS).text or ''
            comment_data['ip'] = comment_node.find('{%s}comment_author_IP' % WP_NS).text

            try:
                user = User.objects.get(username=comment_data['guest_name'])
            except User.DoesNotExist:
                user = MARCUS_GUEST
            comment_data['author'] = user

            comment_data['created'] = datetime.strptime(comment_node.find('{%s}comment_date' % WP_NS).text, '%Y-%m-%d %H:%M:%S')
            if settings.USE_TZ:
                comment_data['created'] = timezone.make_aware(comment_data['created'], timezone.utc)

            comment_approved = comment_node.find('{%s}comment_approved' % WP_NS).text
            if comment_approved == "1":
                comment_data['approved'] = comment_data['created']
            elif comment_approved == "spam":
                comment_data['spam_status'] = "spam"  # What should I write here?

            title = u'Comment #{number} by {author}'.format(number=comment_data['id'], author=comment_data['guest_name'])
            self.write_out(u' > {title}... '.format(title=title))

            for pipeline_path in COMMENT_PIPELINES:
                comment_data = self._get_pipeline(pipeline_path)(data=comment_data).output()

            if not comment_data['text']:
                self.write_out(self.style.NOTICE('SKIPPED (unfilled)\n'))
                return None

            Comment.objects.create(**comment_data)

            self.write_out(self.style.ITEM('OK\n'))

    def _get_pipeline(self, pipeline_path):
        module_name, class_name = pipeline_path.rsplit('.', 1)
        module = import_module(module_name)
        return module.__dict__.get(class_name)



from collections import defaultdict

from django import template
from django.template.defaulttags import url

from marcus import models, utils

register = template.Library()


@register.filter
def astree(items, attribute):
    parent_map = defaultdict(list)
    for item in items:
        parent_map[getattr(item, attribute)].append(item)

    def tree_level(parent):
        for item in parent_map[parent]:
            yield item
            sub_items = list(tree_level(item))
            if sub_items:
                yield sub_items
    return list(tree_level(None))


class TreeNode(template.Node):
    def __init__(self, tree, node_list):
        self.tree = tree
        self.node_list = node_list

    def render(self, context):
        tree = self.tree.resolve(context)

        def pairs(items):
            def dirty(items):
                items = iter(items)
                head = None
                try:
                    while True:
                        item = items.next()
                        if isinstance(item, (list, tuple)):
                            yield head, item
                            head = None
                        else:
                            yield head, None
                            head = item
                except StopIteration:
                    yield head, None
            return ((h, t) for h, t in dirty(items) if h or t)

        def render_item(item, sub_items, level):
            return ''.join([
                '<li>',
                item and self.node_list.render(template.Context({'item': item, 'level': level})) or '',
                sub_items and '<ul>%s</ul>' % ''.join(render_items(sub_items, level + 1)) or '',
                '</li>'
            ])

        def render_items(items, level):
            return ''.join(render_item(h, t, level) for h, t in pairs(items))

        return render_items(tree, 0)


@register.tag
def tree(parser, token):
    bits = token.split_contents()
    if len(bits) != 2:
        raise template.TemplateSyntaxError('"%s" takes one argument: tree-structured list' % bits[0])
    node_list = parser.parse('end' + bits[0])
    parser.delete_first_token()
    return TreeNode(parser.compile_filter(bits[1]), node_list)


class IURLNode(template.Node):
    def __init__(self, urlnode, alternative=False):
        self.urlnode = urlnode
        self.alternative = alternative

    def render(self, context):
        language = context.get('language')
        if self.alternative:
            language = utils.altlanguage(language)
        return utils.iurl(self.urlnode.render(context), language)


@register.tag
def iurl(parser, token):
    return IURLNode(url(parser, token))


@register.tag
def ialturl(parser, token):
    return IURLNode(url(parser, token), True)


@register.filter
def translate(value, language):
    def _translate(v):
        if hasattr(v, '__iter__'):
            return [_translate(i) for i in v]
        else:
            return models.Translation(v, language)
    return _translate(value)



from django.contrib.sites.models import Site
from django.conf import settings


def marcus_context(request):
    return {
        'site': Site.objects.get_current(),
        'MARCUS_TITLE': settings.MARCUS_TITLE,
        'MARCUS_SUBTITLE': settings.MARCUS_SUBTITLE,
    }



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-04-15 10:41
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(blank=True, max_length=200, unique=True)),
                ('title_ru', models.CharField(blank=True, max_length=255)),
                ('text_ru', models.TextField(blank=True)),
                ('title_en', models.CharField(blank=True, max_length=255)),
                ('text_en', models.TextField(blank=True)),
                ('published', models.DateTimeField(blank=True, db_index=True, null=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('comments_hidden', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='ArticleUpload',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('upload', models.FileField(upload_to=b'uploads')),
                ('article', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='uploads', to='marcus.Article')),
            ],
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(blank=True, unique=True)),
                ('title_ru', models.CharField(blank=True, max_length=255)),
                ('description_ru', models.TextField(blank=True)),
                ('count_articles_ru', models.PositiveIntegerField(default=0)),
                ('title_en', models.CharField(blank=True, max_length=255)),
                ('description_en', models.TextField(blank=True)),
                ('count_articles_en', models.PositiveIntegerField(default=0)),
                ('essential', models.BooleanField(db_index=True, default=False)),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='marcus.Category')),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(choices=[(b'comment', 'Comment'), (b'pingback', 'Pingback')], max_length=20)),
                ('text', models.TextField(verbose_name='Text')),
                ('language', models.CharField(choices=[(b'ru', 'Russian'), (b'en', 'English')], max_length=2, verbose_name='Language')),
                ('guest_name', models.CharField(blank=True, max_length=255)),
                ('guest_email', models.CharField(blank=True, default=b'', max_length=200)),
                ('guest_url', models.URLField(blank=True)),
                ('ip', models.GenericIPAddressField(default=b'127.0.0.1')),
                ('spam_status', models.CharField(blank=True, default=b'', max_length=20)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('approved', models.DateTimeField(blank=True, db_index=True, null=True)),
                ('noteworthy', models.BooleanField(default=False)),
                ('followup', models.BooleanField(default=False, help_text='Receive by email further comments in this conversation')),
                ('article', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comments', to='marcus.Article')),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(blank=True, unique=True)),
                ('title_ru', models.CharField(blank=True, max_length=255)),
                ('description_ru', models.TextField(blank=True)),
                ('count_articles_ru', models.PositiveIntegerField(default=0)),
                ('title_en', models.CharField(blank=True, max_length=255)),
                ('description_en', models.TextField(blank=True)),
                ('count_articles_en', models.PositiveIntegerField(default=0)),
            ],
        ),
        migrations.AddField(
            model_name='article',
            name='categories',
            field=models.ManyToManyField(to='marcus.Category'),
        ),
        migrations.AddField(
            model_name='article',
            name='tags',
            field=models.ManyToManyField(to='marcus.Tag'),
        ),
    ]



import markdown2

from django import template
from django.conf import settings

register = template.Library()


@register.filter(is_safe=True)
def markdown(value):
    return markdown2.markdown(value, extras=settings.MARCUS_MARKDOWN_EXTRAS)



# coding: utf-8
from django import template
from django.utils import formats
from django.contrib.humanize.templatetags import humanize
from django.utils.encoding import force_unicode

register = template.Library()


def get_formatted(value, formater):
    if value:
        naturalday = humanize.naturalday(value)
        localize = force_unicode(formats.localize(value, use_l10n=True))
        if formater:
            dtm = formater.format(localize=localize, naturalday=naturalday)
        else:
            dtm = naturalday
        return localize if naturalday.find(unicode(value.year)) != -1 else dtm
    return value


@register.filter(is_safe=False)
def naturaldate(value, formater=None):
    if value:
        value = value.date()
    return unicode(get_formatted(value, formater)).capitalize()



from django.utils.translation import ugettext as _
from django.utils import timezone


def make_approved(modeladmin, request, queryset):
    for item in queryset:
        item.approved = timezone.now()
        item.save()
make_approved.allow_tags = True
make_approved.short_description = _("Make approved")



from django.db.models import signals


def create_system_user(username):
    from django.contrib.auth.models import User
    try:
        User.objects.get(username=username)
    except User.DoesNotExist:
        password = User.objects.make_random_password()
        User.objects.create_user(username, username + '@localhost', password)


def init(sender, **kwargs):
    import marcus.models

    if kwargs['app_config'].models_module == marcus.models:
        create_system_user('marcus_guest')

signals.post_migrate.connect(init)



import django.contrib.sitemaps.views as sitemap_views
from django.views.decorators.cache import cache_page
from django.conf.urls import patterns, url

from marcus.sitemaps import sitemaps


urlpatterns = patterns(
    '',
    url(
        r'^\.xml$',
        cache_page(3600)(sitemap_views.index),
        {'sitemaps': sitemaps, 'sitemap_url_name': 'sitemaps'}),
    url(
        r'^-(?P<section>.+)\.xml$',
        cache_page(3600)(sitemap_views.sitemap),
        {'sitemaps': sitemaps},
        name='sitemaps'),
)



from django.conf.urls import patterns, include, url

from marcus import views, feeds


urlpatterns = patterns(
    '',
    url(r'^(?:(en|ru)/)?$', views.index, name='marcus-index'),

    url(r'^category/(?:(en|ru)/)?$', views.category_index, name='marcus-categories'),
    url(r'^category/([A-Za-z0-9_-]+)/(?:(en|ru)/)?$', views.category, name='marcus-category'),
    url(r'^tag/(?:(en|ru)/)?$', views.tag_index, name='marcus-tags'),
    url(r'^tag/([A-Za-z0-9_-]+)/(?:(en|ru)/)?$', views.tag, name='marcus-tag'),
    url(r'^archive/(?:(en|ru)/)?$', views.archive_index, name='marcus-archive-index'),
    url(r'^archive/(\d{4})/(?:(\d{1,2})/)?(?:(en|ru)/)?$', views.archive, name='marcus-archive'),
    url(r'^(\d{4})/(?:(en|ru)/)?$', views.archive_short, name="marcus-archive-short"),

    url(r'^suspected/$', views.spam_queue, name='marcus-spam-queue'),
    url(r'^suspected/approve/(\d+)/$', views.approve_comment, name='marcus-approve-comment'),
    url(r'^suspected/spam/(\d+)/$', views.spam_comment, name='marcus-spam-comment'),
    url(r'^suspected/delete/$', views.delete_spam, name='marcus-delete-spam'),

    url(r'^feed/(?:(en|ru)/)?$', feeds.Article(), name='marcus-feed'),
    url(r'^category/([A-Za-z0-9_-]+)/feed/(?:(en|ru)/)?$', feeds.Category(), name='marcus-category-feed'),
    url(r'^tag/([A-Za-z0-9_-]+)/feed/(?:(en|ru)/)?$', feeds.Tag(), name='marcus-tag-feed'),
    url(r'^comments/feed/(?:(en|ru)/)?$', feeds.Comment(), name='marcus-comments-feed'),
    url(r'^(\d{4})/(\d{1,2})/(\d{1,2})/([^/]+)/feed/(?:(en|ru)/)?$', feeds.ArticleComment(), name='marcus-article-comments-feed'),
    url(r'^(\d{4})/([^/]+)/feed/(?:(en|ru)/)?$', feeds.ArticleCommentShort(), name="marcus-article-comments-feed-short"),
    url(r'^comments/unsubscribe/(\d+)/(\w+)/(?:(en|ru)/)?$', views.article_comments_unsubscribe, name="marcus-article-comments-unsubscribe"),
    url(r'^comment_preview/$', views.comment_preview, name='marcus-comment-preview'),

    url(r'^(\d{4})/(\d{1,2})/(\d{1,2})/([^/]+)/(?:(en|ru)/)?$', views.article, name='marcus-article'),
    url(r'^(\d{4})/([^/]+)/(?:(en|ru)/)?$', views.article_short, name='marcus-article-short'),
    url(r'^draft/(\d+)/(?:(en|ru)/)?$', views.draft, name='marcus-draft'),

    url(r'^search/(?:(en|ru)/)?$', views.search, name="marcus-search"),
    url(r'^([^/]+)/$', views.find_article, name='marcus-find-article'),

    url(r'^articleuploadimage/preview/(\d+)/$', views.article_upload_image_preview, name="article-upload-image-preview"),
    url(r'^sitemap', include('marcus.sitemap_urls')),

    url(r'^', include('scipio.urls')),
)



import re
import os
import sys

from setuptools import setup, find_packages
from setuptools.command.install_lib import install_lib as _install_lib
from distutils.command.build import build as _build
from distutils.cmd import Command

from marcus import __version__


class compile_translations(Command):
    description = 'compile message catalogs to MO files via django compilemessages'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from django.core.management.commands.compilemessages import Command as CompileMessages

        curdir = os.getcwd()
        os.chdir(os.path.join(os.path.dirname(__file__), 'marcus'))
        cmd = CompileMessages()
        cmd.stdout = sys.stdout
        cmd.stderr = sys.stderr
        cmd.handle(verbosity=4)
        os.chdir(curdir)


class build(_build):
    sub_commands = [('compile_translations', None)] + _build.sub_commands


class install_lib(_install_lib):

    def run(self):
        self.run_command('compile_translations')
        _install_lib.run(self)


# Installation a packages from "requirements.txt"
requirements = open('requirements.txt')
install_requires = []
dependency_links = []
setup_requires = []
try:
    for line in requirements.readlines():
        line = line.strip()
        if line and not line.startswith('#'):  # for inline comments
            if "#egg" in line:
                names = re.findall('#egg=([^-]+)-?', line)
                install_requires.append(names[0])
                links = [link for link in line.split() if '://' in link]
                dependency_links.append(links[0])
            else:
                install_requires.append(line)
                if "Django" in line:
                    setup_requires.append(line)
finally:
    requirements.close()

# Getting long_description
long_description = ""
try:
    readme = open("README.rst")
    long_description = str(readme.read())
    readme.close()
except:
    pass

setup(
    name='django-marcus',
    version=__version__,
    description="Bilingual blog on Django",
    long_description=long_description,
    keywords='django, blog',
    author='Mikhail Andreev',
    author_email='x11org@gmail.com',
    url='http://github.com/adw0rd/marcus',
    license='BSD',
    packages=find_packages(),
    zip_safe=False,
    install_requires=install_requires,
    dependency_links=dependency_links,
    setup_requires=setup_requires,
    package_data={'': ['requirements.txt']},
    include_package_data=True,
    classifiers=[
        "Environment :: Web Environment",
        "Programming Language :: Python",
        "Framework :: Django",
        "License :: OSI Approved :: BSD License",
        "Topic :: Software Development :: Debuggers",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    cmdclass={
        'build': build,
        'install_lib': install_lib,
        'compile_translations': compile_translations
    }
)



from django import forms
from django.contrib import admin

try:
    from markitup.widgets import AdminMarkItUpWidget
    wysiwyg_widget = AdminMarkItUpWidget
except ImportError:
    wysiwyg_widget = forms.Textarea

from marcus import models
from marcus import actions
from marcus import widgets
from marcus import mixins


class TimedBooleanFilter(admin.FieldListFilter):
    def __init__(self, field, request, params, model, model_admin, field_path):
        self.lookup_kwarg = '%s__isnull' % field.name
        self.lookup_val = request.GET.get(self.lookup_kwarg, None)
        super(TimedBooleanFilter, self).__init__(field, request, params, model, model_admin, field_path)

    def expected_parameters(self):
        return [self.lookup_kwarg]

    def choices(self, cl):
        for k, v in [('All', None), ('Yes', ''), ('No', '1')]:
            yield {
                'selected': self.lookup_val == v,
                'query_string': cl.get_query_string({self.lookup_kwarg: v}, []),
                'display': k,
            }


admin.site.register(
    models.Category,
    list_display=('title_ru', 'title_en', 'essential', 'parent', ))

admin.site.register(
    models.Tag,
    list_display=('slug', 'title_ru', 'count_articles_ru', 'title_en', 'count_articles_en', ),
    search_fields=('title_ru', 'title_en', ))


class ArticleUploadForm(forms.ModelForm):
    upload = forms.FileField(widget=widgets.AdminImageWidget)


class ArticleUploadInlineAdmin(admin.TabularInline):
    model = models.ArticleUpload
    form = ArticleUploadForm


class ArticleForm(forms.ModelForm):
    class Meta:
        model = models.Article
        fields = '__all__'
        widgets = {
            'title_ru': forms.TextInput(attrs={'size': 80}),
            'text_ru': wysiwyg_widget(attrs={'cols': 80, 'rows': 30}),
            'title_en': forms.TextInput(attrs={'size': 80}),
            'text_en': wysiwyg_widget(attrs={'cols': 80, 'rows': 30}),
        }

    def clean(self):
        cleaned_data = super(ArticleForm, self).clean()
        title_ru = cleaned_data['title_ru']
        title_en = cleaned_data['title_en']
        text_ru = cleaned_data['text_ru']
        text_en = cleaned_data['text_en']

        if (not title_ru and not text_ru) and (not title_en and not text_en):
            raise forms.ValidationError("Need to fill 'Title ru' or 'Title en'")

        if not title_ru and text_ru:
            self._errors['title_ru'] = self.error_class(["Need to fill 'Title ru'"])
            del cleaned_data['title_ru']

        if not title_en and text_en:
            self._errors['title_en'] = self.error_class(["Need to fill 'Title en'"])
            del cleaned_data['title_en']

        return cleaned_data


class ArticleAdmin(mixins.ArticleTextSizeAdminMixin, admin.ModelAdmin):
    form = ArticleForm
    save_on_top = True
    list_display = ('slug', 'title', 'text_size', 'is_published', )
    list_filter = (('published', TimedBooleanFilter), )
    search_fields = ('slug', 'title_ru', 'title_en', 'text_ru', 'text_en', 'categories__slug', 'categories__title_ru', 'categories__title_en', )
    ordering = ('-published', )
    inlines = (ArticleUploadInlineAdmin, )

    fields = ('slug', 'title_ru', 'text_ru', 'title_en', 'text_en', 'categories', 'tags', 'comments_hidden', 'published', )
    filter_horizontal = ('categories', 'tags', )

    def is_published(self, obj):
        return bool(obj.published)
    is_published.boolean = True

admin.site.register(models.Article, ArticleAdmin)


class ArticleUploadAdmin(admin.ModelAdmin):
    pass

admin.site.register(models.ArticleUpload, ArticleUploadAdmin)


class CommentAdmin(admin.ModelAdmin):
    save_on_top = True
    actions = (actions.make_approved, )
    list_display = ('pk', 'article', 'author_str', 'type', 'created_str', 'is_approved', )
    list_filter = (('approved', TimedBooleanFilter), )
    ordering = ('-created', )
    select_related = ('article', )
    raw_id_fields = ('author', 'article', )
    search_fields = ('article__slug', 'author__username', 'author__scipio_profile__openid', )

    def is_approved(self, obj):
        return bool(obj.approved)
    is_approved.boolean = True

    def created_str(self, obj):
        return obj.created.strftime('%Y-%m-%d\xc2\xa0%H:%M')
    created_str.admin_order_field = 'created'
    created_str.short_description = 'created'

admin.site.register(models.Comment, CommentAdmin)



# coding: utf-8
import re

from django.http import Http404
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.template import loader, Context
from django.shortcuts import get_object_or_404 as goo404
from django.core.mail import EmailMultiAlternatives
from django.utils import translation
from django.utils import timezone


def absolute_url(url):
    if url is None or url.startswith('http://') or url.startswith('https://'):
        return url
    return 'http://%s%s' % (Site.objects.get_current().domain, url)


def iurl(url, language):
    return url if not language else '%s%s/' % (url, language)


def altlanguage(language):
    return {'en': None, 'ru': 'en', None: 'en'}.get(language, language)


def get_language_code_in_text(text):
    """Depending of the used language in the text determines the language,
    currently does not include text quotation.

    @return "en" or "ru"
    """
    language = None
    if text:
        language = "ru" if re.search(u'[а-яА-Я]+', text) else "en"
    return language


def email_text_quote(text, width=10, prefix="|"):
    chunks = []
    text = re.sub(r'\r', r'', text, re.UNICODE)
    sections = re.findall(r'[^\n]+', text, re.UNICODE)
    for section in sections:
        words = re.findall(r'[^\s]+', section, re.UNICODE)
        while words:
            chunk = words[0:width]
            del words[0:width]
            chunks.append(u"{0} {1}".format(prefix, u" ".join(chunk)))
        chunks.append(prefix)
    return u"\n".join(chunks)


def notify_comment_context(target_comment):
    """Build context for comment
    """
    article = target_comment.article
    site = Site.objects.get_current()
    language = 'en' if article.title_en else 'ru'
    translation.activate(language)
    context = {
        'article_title': article.title(language),
        'article_link': article.full_link(language),
        'article_url': article.get_absolute_url(language),
        'site_name': site.name,
        'site_domain': site.domain,
        'sign': settings.MARCUS_TITLE,
        'comment_guest_name': target_comment.guest_name or target_comment.author.username,
        'comment_text': email_text_quote(target_comment.text, width=10, prefix="|"),
        'comment_text_html': target_comment.text,
        'comment_created': target_comment.created,
        'language': language,
    }
    return context


def notify_comment_managers(target_comment):
    """Notify MANAGERS of the blog
    """
    recipients = dict(settings.MANAGERS).values()
    assert recipients, 'Please setup settings.MANAGERS for notify about new comments'

    email = target_comment.guest_email or target_comment.author.email
    if email in recipients:
        recipients.remove(email)
    if not recipients:
        return None

    common_context = notify_comment_context(target_comment)
    common_context['comment_admin_url'] = target_comment.get_admin_url()
    context = Context(common_context)
    text_message = loader.get_template("marcus/emails/managers_comment.txt").render(context)
    html_message = loader.get_template("marcus/emails/managers_comment.html").render(context)
    subject = loader.get_template("marcus/emails/managers_comment_subject.txt").render(context).strip()
    return send_email(subject, text_message, html_message, recipients)


def notify_comment_followers(target_comment):
    """Notify followers of the article comments
    """
    common_context = notify_comment_context(target_comment)
    from marcus.models import Comment

    try:
        comments = Comment.public\
            .filter(article=target_comment.article, followup=True)\
            .exclude(guest_email='')\
            .exclude(guest_email=target_comment.guest_email)\
            .distinct('guest_email')\
            .order_by('guest_email')
        comments = list(comments)
    except NotImplementedError:
        # Temporary fix for MySQL
        comments = Comment.public\
            .filter(article=target_comment.article, followup=True)\
            .exclude(guest_email='')\
            .exclude(guest_email=target_comment.guest_email)
        comments.query.group_by = ['guest_email']

    for comment in comments:
        if not comment.guest_email:
            continue
        # Build context
        unsubscribe_url = reverse(
            "marcus-article-comments-unsubscribe",
            args=[target_comment.article_id, comment.make_token(), common_context['language']]
        )
        context = {
            'unsubscribe_url': unsubscribe_url,
        }
        context.update(common_context)
        context = Context(context)
        # Build message
        text_message = loader.get_template("marcus/emails/followup_comment.txt").render(context)
        html_message = loader.get_template("marcus/emails/followup_comment.html").render(context)
        subject = loader.get_template("marcus/emails/followup_comment_subject.txt").render(context).strip()
        # Send message
        send_email(subject, text_message, html_message, [comment.guest_email])
    return True


def send_email(subject, text_message, html_message, emails):
    message = EmailMultiAlternatives(
        subject, text_message, settings.DEFAULT_FROM_EMAIL, emails)
    message.attach_alternative(html_message, "text/html")
    return message.send()


def get_object_or_404(*args, **kwargs):
    try:
        return goo404(*args, **kwargs)
    except Http404:
        # Hack for old version of Django for saved datetimes in DB.
        # Migration is not safity, so I can't create it.
        timezone.activate(timezone.utc)
        obj = goo404(*args, **kwargs)
        timezone.deactivate()
        return obj



class BasePipeline(object):
    replaced_data = {}

    def __init__(self, data):
        self.data = data
        self.replaced_data = {}

    def output(self):
        for field in self.data:
            replacer_name = 'replace_{field}'.format(field=field)
            if hasattr(self, replacer_name):
                self.replaced_data[field] = getattr(self, replacer_name)(self.data[field])
            else:
                self.replaced_data[field] = self.data[field]
        return self.replaced_data
